
/* Chrysalide - Outil d'analyse de fichiers binaires
 * block.h - prototypes pour l'encadrement des instructions par blocs
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_BLOCK_H
#define _ANALYSIS_BLOCK_H


#include <glib.h>
#include <glib-object.h>
#include <stdbool.h>


#include "../arch/vmpa.h"



#define G_TYPE_INSTR_BLOCK               g_instr_block_get_type()
#define G_INSTR_BLOCK(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_instr_block_get_type(), GInstrBlock))
#define G_IS_INSTR_BLOCK(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_instr_block_get_type()))
#define G_INSTR_BLOCK_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_INSTR_BLOCK, GInstrBlockClass))
#define G_IS_INSTR_BLOCK_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_INSTR_BLOCK))
#define G_INSTR_BLOCK_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_INSTR_BLOCK, GInstrBlockClass))


/* Description d'un bloc d'instructions (instance) */
typedef struct _GInstrBlock GInstrBlock;

/* Description d'un bloc d'instructions (classe) */
typedef struct _GInstrBlockClass GInstrBlockClass;


/* Position au cours d'une visite */
typedef enum _BlockVisitOrder
{
    BVO_IN,                                 /* Entrée dans un gros bloc    */
    BVO_PENDING,                            /* Visite d'un bloc simple     */
    BVO_OUT                                 /* Sortie d'un gros bloc       */

} BlockVisitOrder;

/* Rappel à chaque bloc visité */
typedef bool (* instr_block_visitor_cb) (GInstrBlock *, BlockVisitOrder, void *);


/* Indique le type défini pour un bloc d'instructions. */
GType g_instr_block_get_type(void);

/* Recherche le bloc contenant une adresse donnée. */
GInstrBlock *g_instr_block_find_by_addr(const GInstrBlock *, const vmpa2t *, bool);

/* Parcourt tous les blocs d'instructions dans un ordre donné. */
bool g_instr_block_visit(GInstrBlock *, instr_block_visitor_cb, void *);

/* Etablit la liste de tous les blocs présents. */
void g_instr_block_list_all_blocks(const GInstrBlock *, GInstrBlock ***, size_t *);

/* Etablit la liste de tous les blocs terminaux. */
void g_instr_block_list_leafs_blocks(const GInstrBlock *, GInstrBlock ***, size_t *);

/* Définit l'ensemble contenant les blocs liés. */
void g_instr_block_set_links_block(GInstrBlock *, GInstrBlock *);

/* Fournit l'ensemble contenant les blocs liés. */
GInstrBlock *g_instr_block_get_links_block(const GInstrBlock *);



#endif  /* _ANALYSIS_BLOCK_H */
