
/* Chrysalide - Outil d'analyse de fichiers binaires
 * gbufferview.h - prototypes pour l'affichage d'une vue particulière d'un tampon de lignes
 *
 * Copyright (C) 2016 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _GLIBEXT_GBUFFERVIEW_H
#define _GLIBEXT_GBUFFERVIEW_H


#include <glib-object.h>


#include "gcodebuffer.h"



#define G_TYPE_BUFFER_VIEW              (g_buffer_view_get_type())
#define G_BUFFER_VIEW(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_BUFFER_VIEW, GBufferView))
#define G_BUFFER_VIEW_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_BUFFER_VIEW, GBufferViewClass))
#define G_IS_BUFFER_VIEW(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_BUFFER_VIEW))
#define G_IS_BUFFER_VIEW_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_BUFFER_VIEW))
#define G_BUFFER_VIEW_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_BUFFER_VIEW, GBufferViewClass))


/* Vue d'un tampon pour code désassemblé (instance) */
typedef struct _GBufferView GBufferView;

/* Vue d'un tampon pour code désassemblé (classe) */
typedef struct _GBufferViewClass GBufferViewClass;


/* Détermine le type de la vue d'un tampon pour code désassemblé. */
GType g_buffer_view_get_type(void);

/* Crée une nouvelle vue d'un tampon pour code désassemblé. */
GBufferView *g_buffer_view_new(GCodeBuffer *, segcnt_list *);

/* Restreint le champ d'application de l'affichage. */
void g_buffer_view_restrict(GBufferView *, const vmpa2t *, const vmpa2t *);

/* Indique le champ d'application de l'affichage. */
bool g_buffer_view_get_restrictions(GBufferView *, vmpa2t *, vmpa2t *);

/* Fournit le tampon de code lié à un visualisateur donné. */
GCodeBuffer *g_buffer_view_get_buffer(const GBufferView *);

/* Fournit la hauteur d'impression d'une ligne visualisée. */
gint g_buffer_view_get_line_height(GBufferView *);

/* Fournit la largeur requise par une visualisation. */
gint g_buffer_view_get_width(GBufferView *, const bool *);

/* Fournit la largeur requise pour dépasser les marges gauches. */
gint g_buffer_view_get_margin(GBufferView *, const bool *);

/* Fournit la hauteur requise par une visualisation. */
gint g_buffer_view_get_height(const GBufferView *);

/* Calcule la position idéale de curseur pour un point donné. */
const vmpa2t *g_buffer_view_compute_caret(GBufferView *, gint, gint, const bool *, GdkRectangle *);

/* Calcule la position idéale de curseur pour un point donné. */
const vmpa2t *g_buffer_view_compute_caret_full(GBufferView *, GBufferLine *, size_t, gint, const bool *, GdkRectangle *);

/* Déplace le curseur au sein d'une vue de tampon. */
const vmpa2t *g_buffer_view_move_caret(GBufferView *, GdkRectangle *, bool, GdkScrollDirection, const bool *);

/* Supprime toute mise en évidence de segments. */
bool g_buffer_view_unhighlight_segments(GBufferView *);

/* Surligne tous les segments similaires à celui sous la souris. */
bool g_buffer_view_highlight_segments(GBufferView *, gint, gint, const bool *);

/* Imprime la visualisation du tampon de code désassemblé. */
void g_buffer_view_draw(const GBufferView *, cairo_t *, gint, gint, const cairo_rectangle_int_t *, const bool *, const gint *);

/* Retrouve une ligne au sein d'un tampon avec une adresse. */
GBufferLine *g_buffer_view_find_line_by_addr(const GBufferView *, const vmpa2t *, BufferLineFlags, size_t *);

/* Retrouve une ligne au sein d'un tampon avec un indice. */
GBufferLine *g_buffer_view_find_line_by_index(const GBufferView *, size_t );

/* Fournit la ligne présente à une ordonnée donnée. */
GBufferLine *g_buffer_view_find_line_at(GBufferView *, gint, size_t *);

/* Fournit la ligne et son segment présents à une position. */
GBufferLine *g_buffer_view_find_line_and_segment_at(GBufferView *, gint *, gint, size_t *, const bool *, GBufferSegment **);

/* Indique la position d'affichage d'une adresse donnée. */
bool g_buffer_view_get_address_coordinates(GBufferView *, const vmpa2t *, gint *, gint *, bool);



#endif  /* _GLIBEXT_GBUFFERVIEW_H */
